/*
 *
 * Copyright (C) 2008 Tomasz Ostrowski ( http://ostry.w.szu.pl )
 *
 * This file is part of Free GG.
 *
 * Free GG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Free GG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Free GG; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef PluginManagerH
#define PluginManagerH

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define MAX_PLUGIN_COUNT 256

typedef int (*MSG_CALLBACK)(char*,char*);

typedef int (*PROC_LOAD)(MSG_CALLBACK);
typedef int (*PROC_UNLOAD)();
typedef char* (*PROC_NAME)();

typedef int (*PROC_HANDLEMSG)(char*);

typedef struct
{
    HINSTANCE hInstance;
    
    PROC_NAME Name;
    PROC_LOAD Load;
    PROC_UNLOAD Unload;
    PROC_HANDLEMSG HandleMessage;

} PLUGIN;

typedef struct _plugin_msg
{
    char*           PluginName;
    char*           Message;
    _plugin_msg*    NextMessage;
    
} PLUGIN_MSG;

int QueueMessage(char* plugin, char* message);

class PluginManager
{
public:
	PluginManager::PluginManager(char* dirname, char* inidirname);
	PluginManager::~PluginManager();

	PLUGIN  PluginList[MAX_PLUGIN_COUNT];
	int     PluginCount;	

	int LoadPluginsFromDir(char* dirname);
	int Run(void);
	void ShowPlugin(int iID);
	int DestroyPlugins();
	int HandleNextMessage();
	void PostMsg(char* msg);
private:

	DWORD dwtid;
	int LoadPlugin(char* filename);
	inline bool StartsWith(char *&a, char *b);
};
DWORD WINAPI MessageThreadProc(LPVOID data);

#endif
